@@ -258,7 +258,6 @@
 #define _LGPL_SOURCE
 #include <stdlib.h>
 #include <errno.h>
-#include <assert.h>
 #include <stdio.h>
 #include <stdint.h>
 #include <string.h>
@@ -266,6 +265,7 @@
 #include <unistd.h>
 
 #include "compat-getcpu.h"
+#include <urcu/assert.h>
 #include <urcu/pointer.h>
 #include <urcu/call-rcu.h>
 #include <urcu/flavor.h>
@@ -274,10 +274,10 @@
 #include <urcu/compiler.h>
 #include <urcu/rculfhash.h>
 #include <urcu/static/urcu-signal-nr.h>
-#include <rculfhash-internal.h>
 #include <stdio.h>
 #include <pthread.h>
 #include <signal.h>
+#include "rculfhash-internal.h"
 #include "workqueue.h"
 #include "urcu-die.h"
 #include "urcu-utils.h"
@@ -363,7 +363,6 @@ struct partition_resize_work {
 };
 
 static struct urcu_workqueue *cds_lfht_workqueue;
-static unsigned long cds_lfht_workqueue_user_count;
 
 /*
  * Mutex ensuring mutual exclusion between workqueue initialization and
@@ -380,8 +379,8 @@ static struct urcu_atfork cds_lfht_atfork;
  */
 static int cds_lfht_workqueue_atfork_nesting;
 
+static void __attribute__((destructor)) cds_lfht_exit(void);
 static void cds_lfht_init_worker(const struct rcu_flavor_struct *flavor);
-static void cds_lfht_fini_worker(const struct rcu_flavor_struct *flavor);
 
 #ifdef CONFIG_CDS_LFHT_ITER_DEBUG
 
@@ -391,7 +390,7 @@ void cds_lfht_iter_debug_set_ht(struct cds_lfht *ht, struct cds_lfht_iter *iter)
 	iter->lfht = ht;
 }
 
-#define cds_lfht_iter_debug_assert(...)		assert(__VA_ARGS__)
+#define cds_lfht_iter_debug_assert(...)		urcu_posix_assert(__VA_ARGS__)
 
 #else
 
@@ -676,12 +675,12 @@ void alloc_split_items_count(struct cds_lfht *ht)
 			cds_lfht_get_count_order_ulong(split_count_mask + 1);
 	}
 
-	assert(split_count_mask >= 0);
+	urcu_posix_assert(split_count_mask >= 0);
 
 	if (ht->flags & CDS_LFHT_ACCOUNTING) {
 		ht->split_count = calloc(split_count_mask + 1,
 					sizeof(struct ht_items_count));
-		assert(ht->split_count);
+		urcu_posix_assert(ht->split_count);
 	} else {
 		ht->split_count = NULL;
 	}
@@ -698,7 +697,7 @@ int ht_get_split_count_index(unsigned long hash)
 {
 	int cpu;
 
-	assert(split_count_mask >= 0);
+	urcu_posix_assert(split_count_mask >= 0);
 	cpu = urcu_sched_getcpu();
 	if (caa_unlikely(cpu < 0))
 		return hash & split_count_mask;
@@ -845,6 +844,12 @@ int is_removal_owner(struct cds_lfht_node *node)
 	return ((unsigned long) node) & REMOVAL_OWNER_FLAG;
 }
 
+static
+struct cds_lfht_node *flag_removed(struct cds_lfht_node *node)
+{
+	return (struct cds_lfht_node *) (((unsigned long) node) | REMOVED_FLAG);
+}
+
 static
 struct cds_lfht_node *flag_removal_owner(struct cds_lfht_node *node)
 {
@@ -911,7 +916,7 @@ static inline
 struct cds_lfht_node *lookup_bucket(struct cds_lfht *ht, unsigned long size,
 		unsigned long hash)
 {
-	assert(size > 0);
+	urcu_posix_assert(size > 0);
 	return bucket_at(ht, hash & (size - 1));
 }
 
@@ -923,26 +928,26 @@ void _cds_lfht_gc_bucket(struct cds_lfht_node *bucket, struct cds_lfht_node *nod
 {
 	struct cds_lfht_node *iter_prev, *iter, *next, *new_next;
 
-	assert(!is_bucket(bucket));
-	assert(!is_removed(bucket));
-	assert(!is_removal_owner(bucket));
-	assert(!is_bucket(node));
-	assert(!is_removed(node));
-	assert(!is_removal_owner(node));
+	urcu_posix_assert(!is_bucket(bucket));
+	urcu_posix_assert(!is_removed(bucket));
+	urcu_posix_assert(!is_removal_owner(bucket));
+	urcu_posix_assert(!is_bucket(node));
+	urcu_posix_assert(!is_removed(node));
+	urcu_posix_assert(!is_removal_owner(node));
 	for (;;) {
 		iter_prev = bucket;
 		/* We can always skip the bucket node initially */
 		iter = rcu_dereference(iter_prev->next);
-		assert(!is_removed(iter));
-		assert(!is_removal_owner(iter));
-		assert(iter_prev->reverse_hash <= node->reverse_hash);
+		urcu_posix_assert(!is_removed(iter));
+		urcu_posix_assert(!is_removal_owner(iter));
+		urcu_posix_assert(iter_prev->reverse_hash <= node->reverse_hash);
 		/*
 		 * We should never be called with bucket (start of chain)
 		 * and logically removed node (end of path compression
 		 * marker) being the actual same node. This would be a
 		 * bug in the algorithm implementation.
 		 */
-		assert(bucket != node);
+		urcu_posix_assert(bucket != node);
 		for (;;) {
 			if (caa_unlikely(is_end(iter)))
 				return;
@@ -954,8 +959,8 @@ void _cds_lfht_gc_bucket(struct cds_lfht_node *bucket, struct cds_lfht_node *nod
 			iter_prev = clear_flag(iter);
 			iter = next;
 		}
-		assert(!is_removed(iter));
-		assert(!is_removal_owner(iter));
+		urcu_posix_assert(!is_removed(iter));
+		urcu_posix_assert(!is_removal_owner(iter));
 		if (is_bucket(iter))
 			new_next = flag_bucket(clear_flag(next));
 		else
@@ -975,13 +980,13 @@ int _cds_lfht_replace(struct cds_lfht *ht, unsigned long size,
 	if (!old_node)	/* Return -ENOENT if asked to replace NULL node */
 		return -ENOENT;
 
-	assert(!is_removed(old_node));
-	assert(!is_removal_owner(old_node));
-	assert(!is_bucket(old_node));
-	assert(!is_removed(new_node));
-	assert(!is_removal_owner(new_node));
-	assert(!is_bucket(new_node));
-	assert(new_node != old_node);
+	urcu_posix_assert(!is_removed(old_node));
+	urcu_posix_assert(!is_removal_owner(old_node));
+	urcu_posix_assert(!is_bucket(old_node));
+	urcu_posix_assert(!is_removed(new_node));
+	urcu_posix_assert(!is_removal_owner(new_node));
+	urcu_posix_assert(!is_bucket(new_node));
+	urcu_posix_assert(new_node != old_node);
 	for (;;) {
 		/* Insert after node to be replaced */
 		if (is_removed(old_next)) {
@@ -991,14 +996,14 @@ int _cds_lfht_replace(struct cds_lfht *ht, unsigned long size,
 			 */
 			return -ENOENT;
 		}
-		assert(old_next == clear_flag(old_next));
-		assert(new_node != old_next);
+		urcu_posix_assert(old_next == clear_flag(old_next));
+		urcu_posix_assert(new_node != old_next);
 		/*
 		 * REMOVAL_OWNER flag is _NEVER_ set before the REMOVED
 		 * flag. It is either set atomically at the same time
 		 * (replace) or after (del).
 		 */
-		assert(!is_removal_owner(old_next));
+		urcu_posix_assert(!is_removal_owner(old_next));
 		new_node->next = old_next;
 		/*
 		 * Here is the whole trick for lock-free replace: we add
@@ -1030,7 +1035,7 @@ int _cds_lfht_replace(struct cds_lfht *ht, unsigned long size,
 	bucket = lookup_bucket(ht, size, bit_reverse_ulong(old_node->reverse_hash));
 	_cds_lfht_gc_bucket(bucket, new_node);
 
-	assert(is_removed(CMM_LOAD_SHARED(old_node->next)));
+	urcu_posix_assert(is_removed(CMM_LOAD_SHARED(old_node->next)));
 	return 0;
 }
 
@@ -1052,9 +1057,9 @@ void _cds_lfht_add(struct cds_lfht *ht,
 			*return_node;
 	struct cds_lfht_node *bucket;
 
-	assert(!is_bucket(node));
-	assert(!is_removed(node));
-	assert(!is_removal_owner(node));
+	urcu_posix_assert(!is_bucket(node));
+	urcu_posix_assert(!is_removed(node));
+	urcu_posix_assert(!is_removal_owner(node));
 	bucket = lookup_bucket(ht, size, hash);
 	for (;;) {
 		uint32_t chain_len = 0;
@@ -1066,7 +1071,7 @@ void _cds_lfht_add(struct cds_lfht *ht,
 		iter_prev = bucket;
 		/* We can always skip the bucket node initially */
 		iter = rcu_dereference(iter_prev->next);
-		assert(iter_prev->reverse_hash <= node->reverse_hash);
+		urcu_posix_assert(iter_prev->reverse_hash <= node->reverse_hash);
 		for (;;) {
 			if (caa_unlikely(is_end(iter)))
 				goto insert;
@@ -1119,12 +1124,12 @@ void _cds_lfht_add(struct cds_lfht *ht,
 		}
 
 	insert:
-		assert(node != clear_flag(iter));
-		assert(!is_removed(iter_prev));
-		assert(!is_removal_owner(iter_prev));
-		assert(!is_removed(iter));
-		assert(!is_removal_owner(iter));
-		assert(iter_prev != node);
+		urcu_posix_assert(node != clear_flag(iter));
+		urcu_posix_assert(!is_removed(iter_prev));
+		urcu_posix_assert(!is_removal_owner(iter_prev));
+		urcu_posix_assert(!is_removed(iter));
+		urcu_posix_assert(!is_removal_owner(iter));
+		urcu_posix_assert(iter_prev != node);
 		if (!bucket_flag)
 			node->next = clear_flag(iter);
 		else
@@ -1142,8 +1147,8 @@ void _cds_lfht_add(struct cds_lfht *ht,
 		}
 
 	gc_node:
-		assert(!is_removed(iter));
-		assert(!is_removal_owner(iter));
+		urcu_posix_assert(!is_removed(iter));
+		urcu_posix_assert(!is_removal_owner(iter));
 		if (is_bucket(iter))
 			new_next = flag_bucket(clear_flag(next));
 		else
@@ -1168,9 +1173,9 @@ int _cds_lfht_del(struct cds_lfht *ht, unsigned long size,
 		return -ENOENT;
 
 	/* logically delete the node */
-	assert(!is_bucket(node));
-	assert(!is_removed(node));
-	assert(!is_removal_owner(node));
+	urcu_posix_assert(!is_bucket(node));
+	urcu_posix_assert(!is_removed(node));
+	urcu_posix_assert(!is_removal_owner(node));
 
 	/*
 	 * We are first checking if the node had previously been
@@ -1181,7 +1186,7 @@ int _cds_lfht_del(struct cds_lfht *ht, unsigned long size,
 	next = CMM_LOAD_SHARED(node->next);	/* next is not dereferenced */
 	if (caa_unlikely(is_removed(next)))
 		return -ENOENT;
-	assert(!is_bucket(next));
+	urcu_posix_assert(!is_bucket(next));
 	/*
 	 * The del operation semantic guarantees a full memory barrier
 	 * before the uatomic_or atomic commit of the deletion flag.
@@ -1204,7 +1209,7 @@ int _cds_lfht_del(struct cds_lfht *ht, unsigned long size,
 	bucket = lookup_bucket(ht, size, bit_reverse_ulong(node->reverse_hash));
 	_cds_lfht_gc_bucket(bucket, node);
 
-	assert(is_removed(CMM_LOAD_SHARED(node->next)));
+	urcu_posix_assert(is_removed(CMM_LOAD_SHARED(node->next)));
 	/*
 	 * Last phase: atomically exchange node->next with a version
 	 * having "REMOVAL_OWNER_FLAG" set. If the returned node->next
@@ -1245,8 +1250,9 @@ void partition_resize_helper(struct cds_lfht *ht, unsigned long i,
 	struct partition_resize_work *work;
 	int ret;
 	unsigned long thread, nr_threads;
+	sigset_t newmask, oldmask;
 
-	assert(nr_cpus_mask != -1);
+	urcu_posix_assert(nr_cpus_mask != -1);
 	if (nr_cpus_mask < 0 || len < 2 * MIN_PARTITION_PER_THREAD)
 		goto fallback;
 
@@ -1267,13 +1273,20 @@ void partition_resize_helper(struct cds_lfht *ht, unsigned long i,
 		dbg_printf("error allocating for resize, single-threading\n");
 		goto fallback;
 	}
+
+	ret = sigfillset(&newmask);
+	urcu_posix_assert(!ret);
+	ret = pthread_sigmask(SIG_BLOCK, &newmask, &oldmask);
+	urcu_posix_assert(!ret);
+
 	for (thread = 0; thread < nr_threads; thread++) {
 		work[thread].ht = ht;
 		work[thread].i = i;
 		work[thread].len = partition_len;
 		work[thread].start = thread * partition_len;
 		work[thread].fct = fct;
-		ret = pthread_create(&(work[thread].thread_id), ht->resize_attr,
+		ret = pthread_create(&(work[thread].thread_id),
+			ht->caller_resize_attr ? &ht->resize_attr : NULL,
 			partition_resize_thread, &work[thread]);
 		if (ret == EAGAIN) {
 			/*
@@ -1286,11 +1299,15 @@ void partition_resize_helper(struct cds_lfht *ht, unsigned long i,
 			nr_threads = thread;
 			break;
 		}
-		assert(!ret);
+		urcu_posix_assert(!ret);
 	}
+
+	ret = pthread_sigmask(SIG_SETMASK, &oldmask, NULL);
+	urcu_posix_assert(!ret);
+
 	for (thread = 0; thread < nr_threads; thread++) {
 		ret = pthread_join(work[thread].thread_id, NULL);
-		assert(!ret);
+		urcu_posix_assert(!ret);
 	}
 	free(work);
 
@@ -1322,12 +1339,12 @@ void init_table_populate_partition(struct cds_lfht *ht, unsigned long i,
 {
 	unsigned long j, size = 1UL << (i - 1);
 
-	assert(i > MIN_TABLE_ORDER);
+	urcu_posix_assert(i > MIN_TABLE_ORDER);
 	ht->flavor->read_lock();
 	for (j = size + start; j < size + start + len; j++) {
 		struct cds_lfht_node *new_node = bucket_at(ht, j);
 
-		assert(j >= size && j < (size << 1));
+		urcu_posix_assert(j >= size && j < (size << 1));
 		dbg_printf("init populate: order %lu index %lu hash %lu\n",
 			   i, j, j);
 		new_node->reverse_hash = bit_reverse_ulong(j);
@@ -1351,7 +1368,7 @@ void init_table(struct cds_lfht *ht,
 
 	dbg_printf("init table: first_order %lu last_order %lu\n",
 		   first_order, last_order);
-	assert(first_order > MIN_TABLE_ORDER);
+	urcu_posix_assert(first_order > MIN_TABLE_ORDER);
 	for (i = first_order; i <= last_order; i++) {
 		unsigned long len;
 
@@ -1414,13 +1431,13 @@ void remove_table_partition(struct cds_lfht *ht, unsigned long i,
 {
 	unsigned long j, size = 1UL << (i - 1);
 
-	assert(i > MIN_TABLE_ORDER);
+	urcu_posix_assert(i > MIN_TABLE_ORDER);
 	ht->flavor->read_lock();
 	for (j = size + start; j < size + start + len; j++) {
 		struct cds_lfht_node *fini_bucket = bucket_at(ht, j);
 		struct cds_lfht_node *parent_bucket = bucket_at(ht, j - size);
 
-		assert(j >= size && j < (size << 1));
+		urcu_posix_assert(j >= size && j < (size << 1));
 		dbg_printf("remove entry: order %lu index %lu hash %lu\n",
 			   i, j, j);
 		/* Set the REMOVED_FLAG to freeze the ->next for gc */
@@ -1449,7 +1466,7 @@ void fini_table(struct cds_lfht *ht,
 
 	dbg_printf("fini table: first_order %lu last_order %lu\n",
 		   first_order, last_order);
-	assert(first_order > MIN_TABLE_ORDER);
+	urcu_posix_assert(first_order > MIN_TABLE_ORDER);
 	for (i = last_order; i >= first_order; i--) {
 		unsigned long len;
 
@@ -1512,7 +1529,7 @@ void cds_lfht_create_bucket(struct cds_lfht *ht, unsigned long size)
 	node->reverse_hash = 0;
 
 	bucket_order = cds_lfht_get_count_order_ulong(size);
-	assert(bucket_order >= 0);
+	urcu_posix_assert(bucket_order >= 0);
 
 	for (order = 1; order < (unsigned long) bucket_order + 1; order++) {
 		len = 1UL << (order - 1);
@@ -1538,7 +1555,7 @@ void cds_lfht_create_bucket(struct cds_lfht *ht, unsigned long size)
 			node->reverse_hash = bit_reverse_ulong(len + i);
 
 			/* insert after prev */
-			assert(is_bucket(prev->next));
+			urcu_posix_assert(is_bucket(prev->next));
 			node->next = prev->next;
 			prev->next = flag_bucket(node);
 		}
@@ -1572,6 +1589,12 @@ const struct cds_lfht_mm_type *get_mm_type(
 }
 #endif
 
+void cds_lfht_node_init_deleted(struct cds_lfht_node *node)
+{
+	cds_lfht_node_init(node);
+	node->next = flag_removed(NULL);
+}
+
 struct cds_lfht *_cds_lfht_new(unsigned long init_size,
 			unsigned long min_nr_alloc_buckets,
 			unsigned long max_nr_buckets,
@@ -1614,13 +1637,15 @@ struct cds_lfht *_cds_lfht_new(unsigned long init_size,
 	init_size = min(init_size, max_nr_buckets);
 
 	ht = mm->alloc_cds_lfht(min_nr_alloc_buckets, max_nr_buckets);
-	assert(ht);
-	assert(ht->mm == mm);
-	assert(ht->bucket_at == mm->bucket_at);
+	urcu_posix_assert(ht);
+	urcu_posix_assert(ht->mm == mm);
+	urcu_posix_assert(ht->bucket_at == mm->bucket_at);
 
 	ht->flags = flags;
 	ht->flavor = flavor;
-	ht->resize_attr = attr;
+	ht->caller_resize_attr = attr;
+	if (attr)
+		ht->resize_attr = *attr;
 	alloc_split_items_count(ht);
 	/* this mutex should not nest in read-side C.S. */
 	pthread_mutex_init(&ht->resize_mutex, NULL);
@@ -1657,7 +1682,7 @@ void cds_lfht_lookup(struct cds_lfht *ht, unsigned long hash,
 			break;
 		}
 		next = rcu_dereference(node->next);
-		assert(node == clear_flag(node));
+		urcu_posix_assert(node == clear_flag(node));
 		if (caa_likely(!is_removed(next))
 		    && !is_bucket(next)
 		    && node->reverse_hash == reverse_hash
@@ -1666,7 +1691,7 @@ void cds_lfht_lookup(struct cds_lfht *ht, unsigned long hash,
 		}
 		node = clear_flag(next);
 	}
-	assert(!node || !is_bucket(CMM_LOAD_SHARED(node->next)));
+	urcu_posix_assert(!node || !is_bucket(CMM_LOAD_SHARED(node->next)));
 	iter->node = node;
 	iter->next = next;
 }
@@ -1701,7 +1726,7 @@ void cds_lfht_next_duplicate(struct cds_lfht *ht __attribute__((unused)),
 		}
 		node = clear_flag(next);
 	}
-	assert(!node || !is_bucket(CMM_LOAD_SHARED(node->next)));
+	urcu_posix_assert(!node || !is_bucket(CMM_LOAD_SHARED(node->next)));
 	iter->node = node;
 	iter->next = next;
 }
@@ -1725,7 +1750,7 @@ void cds_lfht_next(struct cds_lfht *ht __attribute__((unused)),
 		}
 		node = clear_flag(next);
 	}
-	assert(!node || !is_bucket(CMM_LOAD_SHARED(node->next)));
+	urcu_posix_assert(!node || !is_bucket(CMM_LOAD_SHARED(node->next)));
 	iter->node = node;
 	iter->next = next;
 }
@@ -1834,6 +1859,35 @@ int cds_lfht_is_node_deleted(const struct cds_lfht_node *node)
 	return is_removed(CMM_LOAD_SHARED(node->next));
 }
 
+static
+bool cds_lfht_is_empty(struct cds_lfht *ht)
+{
+	struct cds_lfht_node *node, *next;
+	bool empty = true;
+	bool was_online;
+
+	was_online = ht->flavor->read_ongoing();
+	if (!was_online) {
+		ht->flavor->thread_online();
+		ht->flavor->read_lock();
+	}
+	/* Check that the table is empty */
+	node = bucket_at(ht, 0);
+	do {
+		next = rcu_dereference(node->next);
+		if (!is_bucket(next)) {
+			empty = false;
+			break;
+		}
+		node = clear_flag(next);
+	} while (!is_end(node));
+	if (!was_online) {
+		ht->flavor->read_unlock();
+		ht->flavor->thread_offline();
+	}
+	return empty;
+}
+
 static
 int cds_lfht_delete_bucket(struct cds_lfht *ht)
 {
@@ -1846,8 +1900,8 @@ int cds_lfht_delete_bucket(struct cds_lfht *ht)
 		node = clear_flag(node)->next;
 		if (!is_bucket(node))
 			return -EPERM;
-		assert(!is_removed(node));
-		assert(!is_removal_owner(node));
+		urcu_posix_assert(!is_removed(node));
+		urcu_posix_assert(!is_removal_owner(node));
 	} while (!is_end(node));
 	/*
 	 * size accessed without rcu_dereference because hash table is
@@ -1859,7 +1913,7 @@ int cds_lfht_delete_bucket(struct cds_lfht *ht)
 		node = bucket_at(ht, i);
 		dbg_printf("delete bucket: index %lu expected hash %lu hash %lu\n",
 			i, i, bit_reverse_ulong(node->reverse_hash));
-		assert(is_bucket(node->next));
+		urcu_posix_assert(is_bucket(node->next));
 	}
 
 	for (order = cds_lfht_get_count_order_ulong(size); (long)order >= 0; order--)
@@ -1868,6 +1922,24 @@ int cds_lfht_delete_bucket(struct cds_lfht *ht)
 	return 0;
 }
 
+static
+void do_auto_resize_destroy_cb(struct urcu_work *work)
+{
+	struct cds_lfht *ht = caa_container_of(work, struct cds_lfht, destroy_work);
+	int ret;
+
+	ht->flavor->register_thread();
+	ret = cds_lfht_delete_bucket(ht);
+	if (ret)
+		urcu_die(-ret);
+	free_split_items_count(ht);
+	ret = pthread_mutex_destroy(&ht->resize_mutex);
+	if (ret)
+		urcu_die(ret);
+	ht->flavor->unregister_thread();
+	poison_free(ht);
+}
+
 /*
  * Should only be called when no more concurrent readers nor writers can
  * possibly access the table.
@@ -1877,22 +1949,38 @@ int cds_lfht_destroy(struct cds_lfht *ht, pthread_attr_t **attr)
 	int ret;
 
 	if (ht->flags & CDS_LFHT_AUTO_RESIZE) {
+		/*
+		 * Perform error-checking for emptiness before queuing
+		 * work, so we can return error to the caller. This runs
+		 * concurrently with ongoing resize.
+		 */
+		if (!cds_lfht_is_empty(ht))
+			return -EPERM;
 		/* Cancel ongoing resize operations. */
 		_CMM_STORE_SHARED(ht->in_progress_destroy, 1);
-		/* Wait for in-flight resize operations to complete */
-		urcu_workqueue_flush_queued_work(cds_lfht_workqueue);
+		if (attr) {
+			*attr = ht->caller_resize_attr;
+			ht->caller_resize_attr = NULL;
+		}
+		/*
+		 * Queue destroy work after prior queued resize
+		 * operations. Given there are no concurrent writers
+		 * accessing the hash table at this point, no resize
+		 * operations can be queued after this destroy work.
+		 */
+		urcu_workqueue_queue_work(cds_lfht_workqueue,
+			&ht->destroy_work, do_auto_resize_destroy_cb);
+		return 0;
 	}
 	ret = cds_lfht_delete_bucket(ht);
 	if (ret)
 		return ret;
 	free_split_items_count(ht);
 	if (attr)
-		*attr = ht->resize_attr;
+		*attr = ht->caller_resize_attr;
 	ret = pthread_mutex_destroy(&ht->resize_mutex);
 	if (ret)
 		ret = -EBUSY;
-	if (ht->flags & CDS_LFHT_AUTO_RESIZE)
-		cds_lfht_fini_worker(ht->flavor);
 	poison_free(ht);
 	return ret;
 }
@@ -1956,7 +2044,7 @@ void _do_cds_lfht_grow(struct cds_lfht *ht,
 	new_order = cds_lfht_get_count_order_ulong(new_size);
 	dbg_printf("resize from %lu (order %lu) to %lu (order %lu) buckets\n",
 		   old_size, old_order, new_size, new_order);
-	assert(new_size > old_size);
+	urcu_posix_assert(new_size > old_size);
 	init_table(ht, old_order + 1, new_order);
 }
 
@@ -1972,7 +2060,7 @@ void _do_cds_lfht_shrink(struct cds_lfht *ht,
 	new_order = cds_lfht_get_count_order_ulong(new_size);
 	dbg_printf("resize from %lu (order %lu) to %lu (order %lu) buckets\n",
 		   old_size, old_order, new_size, new_order);
-	assert(new_size < old_size);
+	urcu_posix_assert(new_size < old_size);
 
 	/* Remove and unlink all bucket nodes to remove. */
 	fini_table(ht, new_order + 1, old_order);
@@ -2151,51 +2239,24 @@ static struct urcu_atfork cds_lfht_atfork = {
 	.after_fork_child = cds_lfht_after_fork_child,
 };
 
-/*
- * Block all signals for the workqueue worker thread to ensure we don't
- * disturb the application. The SIGRCU signal needs to be unblocked for
- * the urcu-signal flavor.
- */
-static void cds_lfht_worker_init(
-		struct urcu_workqueue *workqueue __attribute__((unused)),
-		void *priv __attribute__((unused)))
-{
-	int ret;
-	sigset_t mask;
-
-	ret = sigfillset(&mask);
-	if (ret)
-		urcu_die(errno);
-	ret = sigdelset(&mask, SIGRCU);
-	if (ret)
-		urcu_die(errno);
-	ret = pthread_sigmask(SIG_SETMASK, &mask, NULL);
-	if (ret)
-		urcu_die(ret);
-}
-
 static void cds_lfht_init_worker(const struct rcu_flavor_struct *flavor)
 {
 	flavor->register_rculfhash_atfork(&cds_lfht_atfork);
 
 	mutex_lock(&cds_lfht_fork_mutex);
-	if (cds_lfht_workqueue_user_count++)
-		goto end;
-	cds_lfht_workqueue = urcu_workqueue_create(0, -1, NULL,
-		NULL, cds_lfht_worker_init, NULL, NULL, NULL, NULL, NULL);
-end:
+	if (!cds_lfht_workqueue)
+		cds_lfht_workqueue = urcu_workqueue_create(0, -1, NULL,
+			NULL, NULL, NULL, NULL, NULL, NULL, NULL);
 	mutex_unlock(&cds_lfht_fork_mutex);
 }
 
-static void cds_lfht_fini_worker(const struct rcu_flavor_struct *flavor)
+static void cds_lfht_exit(void)
 {
 	mutex_lock(&cds_lfht_fork_mutex);
-	if (--cds_lfht_workqueue_user_count)
-		goto end;
-	urcu_workqueue_destroy(cds_lfht_workqueue);
-	cds_lfht_workqueue = NULL;
-end:
+	if (cds_lfht_workqueue) {
+		urcu_workqueue_flush_queued_work(cds_lfht_workqueue);
+		urcu_workqueue_destroy(cds_lfht_workqueue);
+		cds_lfht_workqueue = NULL;
+	}
 	mutex_unlock(&cds_lfht_fork_mutex);
-
-	flavor->unregister_rculfhash_atfork(&cds_lfht_atfork);
 }
